@@ -1,34 +1,25 @@
 /*
-    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
-#include "harness_defs.h"
-
 #define HARNESS_DEFAULT_MIN_THREADS 2
 #define HARNESS_DEFAULT_MAX_THREADS 4
+#include "harness_defs.h"
 
 #if _MSC_VER
     #pragma warning (disable: 4503) // Suppress "decorated name length exceeded, name was truncated" warning
-    #if !TBB_USE_EXCEPTIONS
-        // Suppress "C++ exception handler used, but unwind semantics are not enabled" warning in STL headers
-        #pragma warning (disable: 4530)
-    #endif
 #endif
 
 #if __TBB_MSVC_UNREACHABLE_CODE_IGNORED
@@ -121,15 +112,15 @@ protected:
 // for nodes with limited concurrency, if that concurrency is < g_NumThreads, we need
 // to make sure enough other nodes wait for concurrency to peak.  If we are attached to
 // N successors, for each item we pass to a successor, we will get N executions of the
-// "absorbers" (because we broadcast to successors.)  for an odd number of threads we 
-// need (g_NumThreads - limited + 1) / 2 items (that will give us one extra execution 
+// "absorbers" (because we broadcast to successors.)  for an odd number of threads we
+// need (g_NumThreads - limited + 1) / 2 items (that will give us one extra execution
 // of an "absorber", but we can't change that without changing the behavior of the node.)
 template<>
 class WaitThrow<limited_type,nonThrowing> {
 protected:
     void WaitAndThrow(int cnt, const char * /*name*/) {
         if(cnt <= (g_NumThreads - (int)limited_type + 1)/2) {
-            return; 
+            return;
         }
         Harness::ConcurrencyTracker ct;
         WaitUntilConcurrencyPeaks();
@@ -142,7 +133,7 @@ protected:
     void WaitAndThrow(int cnt, const char * /*name*/) {
         Harness::ConcurrencyTracker ct;
         if(cnt <= (g_NumThreads - (int)limited_type + 1)/2) {
-            return; 
+            return;
         }
         WaitUntilConcurrencyPeaks();
         ThrowTestException(1);
@@ -301,9 +292,9 @@ struct myLess {
 
 // --------- type for < comparison in priority_queue_node.
 template<class ItemType>
-struct less_body : public std::binary_function<ItemType,ItemType,bool> {
+struct less_body {
     bool operator()(const ItemType &lhs, const ItemType &rhs) {
-        return ((int)lhs % 3) < ((int)rhs % 3);
+        return (int(lhs) % 3) < (int(rhs) % 3);
     }
 };
 
@@ -313,8 +304,7 @@ class tag_func {
     TT my_mult;
 public:
     tag_func(TT multiplier) : my_mult(multiplier) { }
-    void operator=( const tag_func& other){my_mult = other.my_mult;}
-    // operator() will return [0 .. Count) 
+    // operator() will return [0 .. Count)
     tbb::flow::tag_value operator()( TT v) {
         tbb::flow::tag_value t = tbb::flow::tag_value(v / my_mult);
         return t;
@@ -409,7 +399,7 @@ void run_one_source_node_test(bool throwException, bool flog) {
             ASSERT(src_cnt == g_NumItems, "Incorrect # source_node items emitted");
             ASSERT(sink_cnt == src_cnt, "Incorrect # source_node items received");
         }
-        g.reset();  // resets the body of the source_node and the absorb_nodes. 
+        g.reset();  // resets the body of the source_node and the absorb_nodes.
         source_body_count = 0;
         absorber_body_count = 0;
         ASSERT(!g.exception_thrown(), "Reset didn't clear exception_thrown()");
@@ -559,14 +549,14 @@ run_one_functype_node_test(bool throwException, bool flog, const char * /*name*/
             ASSERT(t_cnt == 2*g_NumItems, "Not all items reached test node");
             ASSERT(nb0_cnt == 2*g_NumItems && nb1_cnt == 2*g_NumItems, "Missing items in absorbers");
         }
-        g.reset();  // resets the body of the source_nodes, test_node and the absorb_nodes. 
+        g.reset();  // resets the body of the source_nodes, test_node and the absorb_nodes.
         source0_count = source1_count = sink0_count = sink1_count = test_count = 0;
         ASSERT(0 == tbb::flow::copy_body<SourceNodeBodyType0>(source0).count_value(),"Reset source 0 failed");
         ASSERT(0 == tbb::flow::copy_body<SourceNodeBodyType1>(source1).count_value(),"Reset source 1 failed");
         ASSERT(0 == tbb::flow::copy_body<TestNodeBodyType>(node_to_test).count_value(),"Reset test_node failed");
         ASSERT(0 == tbb::flow::copy_body<SinkNodeBodyType0>(sink0).count_value(),"Reset sink 0 failed");
         ASSERT(0 == tbb::flow::copy_body<SinkNodeBodyType1>(sink1).count_value(),"Reset sink 1 failed");
-        
+
         g_Wakeup_Msg = saved_msg;
     }
 #if USE_TASK_SCHEDULER_OBSERVER
@@ -575,7 +565,7 @@ run_one_functype_node_test(bool throwException, bool flog, const char * /*name*/
 }
 
 //  Test function_node
-// 
+//
 // graph being tested is
 //
 //        source_node -\                 /- parallel function_node
@@ -585,8 +575,8 @@ run_one_functype_node_test(bool throwException, bool flog, const char * /*name*/
 //        source_node -/                 \- parallel function_node
 //
 //    After each run the graph is reset(), to test the reset functionality.
-// 
-template< 
+//
+template<
     TestNodeTypeEnum SType1,                          // does source node 1 throw?
     TestNodeTypeEnum SType2,                          // does source node 2 throw?
     class Item12,                                     // type of item passed between sources and test node
@@ -594,7 +584,7 @@ template<
     class Item23,                                     // type passed from function_node to sink nodes
     TestNodeTypeEnum NType1,                          // does sink node 1 throw?
     TestNodeTypeEnum NType2,                          // does sink node 1 throw?
-    tbb::flow::graph_buffer_policy NodePolicy,        // rejecting,queueing
+    class NodePolicy,                                 // rejecting,queueing
     size_t Conc                                       // is node concurrent? {serial | limited | unlimited}
 >
 void run_function_node_test() {
@@ -677,7 +667,7 @@ void test_function_node() {
 
 // ----------------------------------- multifunction_node ----------------------------------
 //  Test multifunction_node.
-// 
+//
 // graph being tested is
 //
 //        source_node -\                      /- parallel function_node
@@ -689,8 +679,8 @@ void test_function_node() {
 //    After each run the graph is reset(), to test the reset functionality.  The
 //    multifunction_node will put an item to each successor for every item
 //    received.
-// 
-template< 
+//
+template<
     TestNodeTypeEnum SType0,                          // does source node 1 throw?
     TestNodeTypeEnum SType1,                          // does source node 2 thorw?
     class Item12,                                 // type of item passed between sources and test node
@@ -698,7 +688,7 @@ template<
     class ItemTuple,                              // tuple of types passed from multifunction_node to sink nodes
     TestNodeTypeEnum NType1,                          // does sink node 1 throw?
     TestNodeTypeEnum NType2,                          // does sink node 2 throw?
-    tbb::flow::graph_buffer_policy NodePolicy,    // rejecting,queueing
+    class  NodePolicy,                            // rejecting,queueing
     size_t Conc                                   // is node concurrent? {serial | limited | unlimited}
 >
 void run_multifunction_node_test() {
@@ -798,7 +788,7 @@ void test_multifunction_node() {
 //
 // The continue_node has unlimited parallelism, no input buffering, and broadcasts to successors.
 // The absorber is parallel, so each item emitted by the source will result in one thread
-// spinning.  So for N threads we pass N-1 continue_messages, then spin wait and then throw if 
+// spinning.  So for N threads we pass N-1 continue_messages, then spin wait and then throw if
 // we are allowed to.
 
 template < class SourceNodeType, class SourceNodeBodyType, class TTestNodeType, class TestNodeBodyType,
@@ -857,7 +847,7 @@ void run_one_continue_node_test (bool throwException, bool flog) {
             ASSERT(t_cnt == g_NumItems, "Not all items reached test node");
             ASSERT(nb_cnt == g_NumItems, "Missing items in absorbers");
         }
-        g.reset();  // resets the body of the source_nodes, test_node and the absorb_nodes. 
+        g.reset();  // resets the body of the source_nodes, test_node and the absorb_nodes.
         source_count = test_count = sink_count = 0;
         ASSERT(0 == (int)test_count, "Atomic wasn't reset properly");
         ASSERT(0 == tbb::flow::copy_body<SourceNodeBodyType>(source).count_value(),"Reset source failed");
@@ -898,7 +888,7 @@ void run_continue_node_test() {
     }
 }
 
-// 
+//
 void test_continue_node() {
     REMARK("Testing continue_node\n");
     g_Wakeup_Msg = "buffer_node(non,is,non): Missed wakeup or machine is overloaded?";
@@ -918,7 +908,7 @@ void test_continue_node() {
 // ---------- buffer_node queue_node overwrite_node --------------
 
 template<
-    class BufferItemType,       // 
+    class BufferItemType,       //
     class SourceNodeType,
     class SourceNodeBodyType,
     class TestNodeType,
@@ -1057,7 +1047,7 @@ void test_buffer_queue_and_overwrite_node() {
 
 
 template<
-    class BufferItemType,       // 
+    class BufferItemType,       //
     class SourceNodeType,
     class SourceNodeBodyType,
     class TestNodeType,
@@ -1297,10 +1287,10 @@ void test_priority_queue_node() {
 }
 
 // ------------------- join_node ----------------
-template<tbb::flow::graph_buffer_policy JP> struct graph_policy_name{
+template<class JP> struct graph_policy_name{
     static const char* name() {return "unknown"; }
 };
-template<> struct graph_policy_name<tbb::flow::queueing>  { 
+template<> struct graph_policy_name<tbb::flow::queueing>  {
     static const char* name() {return "queueing"; }
 };
 template<> struct graph_policy_name<tbb::flow::reserving> {
@@ -1312,14 +1302,14 @@ template<> struct graph_policy_name<tbb::flow::tag_matching> {
 
 
 template<
-    tbb::flow::graph_buffer_policy JP,
+    class JP,
     class OutputTuple,
     class SourceType0,
     class SourceBodyType0,
-    class SourceType1, 
+    class SourceType1,
     class SourceBodyType1,
     class TestJoinType,
-    class SinkType, 
+    class SinkType,
     class SinkBodyType
     >
 struct run_one_join_node_test {
@@ -1415,10 +1405,10 @@ template<
     class OutputTuple,
     class SourceType0,
     class SourceBodyType0,
-    class SourceType1, 
+    class SourceType1,
     class SourceBodyType1,
     class TestJoinType,
-    class SinkType, 
+    class SinkType,
     class SinkBodyType
     >
 struct run_one_join_node_test<
@@ -1430,7 +1420,7 @@ struct run_one_join_node_test<
         SourceBodyType1,
         TestJoinType,
         SinkType,
-        SinkBodyType 
+        SinkBodyType
     > {
     run_one_join_node_test() {}
     static void execute_test(bool throwException,bool flog) {
@@ -1493,7 +1483,7 @@ struct run_one_join_node_test<
                 tbb::flow::input_port<0>(node_to_test).try_put(ItemType0(g_NumItems + 4));
                 tbb::flow::input_port<1>(node_to_test).try_put(ItemType1(g_NumItems + 2));
                 g.wait_for_all();   // have to wait for the graph to stop again....
-                g.reset();  // resets the body of the source_nodes, test_node and the absorb_nodes. 
+                g.reset();  // resets the body of the source_nodes, test_node and the absorb_nodes.
                 source0_count = source1_count = sink_count = 0;
                 make_edge(node_to_test, sink);
                 g.wait_for_all();   // have to wait for the graph to stop again....
@@ -1515,7 +1505,7 @@ struct run_one_join_node_test<
     }
 };  // run_one_join_node_test<tag_matching>
 
-template<tbb::flow::graph_buffer_policy JP, class OutputTuple,
+template<class JP, class OutputTuple,
              TestNodeTypeEnum SourceThrowType,
              TestNodeTypeEnum SinkThrowType>
 void run_join_node_test() {
@@ -1547,7 +1537,7 @@ void run_join_node_test() {
     }
 }
 
-template<tbb::flow::graph_buffer_policy JP>
+template<class JP>
 void test_join_node() {
     REMARK("Testing join_node<%s>\n", graph_policy_name<JP>::name());
     // only doing two-input joins
@@ -1565,7 +1555,7 @@ void test_join_node() {
 // ------------------- limiter_node -------------
 
 template<
-    class BufferItemType,       // 
+    class BufferItemType,       //
     class SourceNodeType,
     class SourceNodeBodyType,
     class TestNodeType,
@@ -1743,7 +1733,7 @@ void run_one_split_node_test(bool throwException, bool flog) {
             ASSERT(sb_cnt == g_NumItems, "Missing invocations of source_nodes");
             ASSERT(nb0_cnt == g_NumItems && nb1_cnt == g_NumItems, "Missing items in absorbers");
         }
-        g.reset();  // resets the body of the source_nodes and the absorb_nodes. 
+        g.reset();  // resets the body of the source_nodes and the absorb_nodes.
         source_count = sink0_count = sink1_count = 0;
         ASSERT(0 == tbb::flow::copy_body<SourceBodyType>(source).count_value(),"Reset source failed");
         ASSERT(0 == tbb::flow::copy_body<SinkBodyType0>(sink0).count_value(),"Reset sink 0 failed");
@@ -1946,7 +1936,7 @@ public:
     }
 };
 
-// test from user ahelwer: http://software.intel.com/en-us/forums/showthread.php?t=103786 
+// test from user ahelwer: http://software.intel.com/en-us/forums/showthread.php?t=103786
 // exception thrown in graph node, not caught in wait_for_all()
 void
 test_flow_graph_exception0() {
@@ -1959,7 +1949,7 @@ test_flow_graph_exception0() {
     // Construct graph and nodes
     tbb::flow::graph g;
     tbb::flow::broadcast_node<tbb::flow::continue_msg> start(g);
-    tbb::flow::continue_node<tbb::flow::continue_msg> fooNode(g, f); 
+    tbb::flow::continue_node<tbb::flow::continue_msg> fooNode(g, f);
 
     // Construct edge
     tbb::flow::make_edge(start, fooNode);
@@ -1999,7 +1989,7 @@ void TestOneThreadNum(int nThread) {
     REMARK("Testing %d threads\n", nThread);
     g_NumItems = ((nThread > NUM_ITEMS) ? nThread *2 : NUM_ITEMS);
     g_NumThreads = nThread;
-    tbb::task_scheduler_init init(nThread); 
+    tbb::task_scheduler_init init(nThread);
     // whole-graph exception catch and rethrow test
     test_flow_graph_exception0();
     for(int i = 0; i < 4; ++i) {
